from abc import ABC
from typing import Optional

class BaseLlmConfig(ABC):
    """
    Config for LLMs.
    """

    def __init__(
        self,
        model: Optional[str] = None,
        temperature: float = 0,
        max_tokens: int = 3000,
        top_p: float = 1
    ):
        """
        Initializes a configuration class instance for the LLM.

        :param model: Controls the OpenAI model used, defaults to None
        :type model: Optional[str], optional
        :param temperature:  Controls the randomness of the model's output.
        Higher values (closer to 1) make output more random, lower values make it more deterministic, defaults to 0
        :type temperature: float, optional
        :param max_tokens: Controls how many tokens are generated, defaults to 3000
        :type max_tokens: int, optional
        :param top_p: Controls the diversity of words. Higher values (closer to 1) make word selection more diverse,
        defaults to 1
        :type top_p: float, optional
        """
        
        self.model = model
        self.temperature = temperature
        self.max_tokens = max_tokens
        self.top_p = top_p


import os
import json
from typing import Dict, List, Optional, Any

try:
    import boto3
except ImportError:
    raise ImportError("AWS Bedrock requires extra dependencies. Install with `pip install boto3`") from None

from mem0.llms.base import LLMBase
from mem0.configs.llms.base import BaseLlmConfig

class AWSBedrockLLM(LLMBase):    
    def __init__(self, config: Optional[BaseLlmConfig] = None):
        super().__init__(config)

        if not self.config.model:
            self.config.model="anthropic.claude-3-5-sonnet-20240620-v1:0"
        self.client = boto3.client("bedrock-runtime", region_name=os.environ.get("AWS_REGION"), aws_access_key_id=os.environ.get("AWS_ACCESS_KEY"), aws_secret_access_key=os.environ.get("AWS_SECRET_ACCESS_KEY"))
        self.model_kwargs = {
            "temperature": self.config.temperature, 
            "max_tokens_to_sample": self.config.max_tokens, 
            "top_p": self.config.top_p
        }

    def _format_messages(self, messages: List[Dict[str, str]]) -> str:
        """
        Formats a list of messages into the required prompt structure for the model.

        Args:
            messages (List[Dict[str, str]]): A list of dictionaries where each dictionary represents a message. 
                                            Each dictionary contains 'role' and 'content' keys.

        Returns:
            str: A formatted string combining all messages, structured with roles capitalized and separated by newlines.
        """
        formatted_messages = []
        for message in messages:
            role = message['role'].capitalize()
            content = message['content']
            formatted_messages.append(f"\n\n{role}: {content}")
        
        return "".join(formatted_messages) + "\n\nAssistant:"
    
    def _parse_response(self, response, tools) -> str:
        """
        Process the response based on whether tools are used or not.

        Args:
            response: The raw response from API.
            tools: The list of tools provided in the request.

        Returns:
            str or dict: The processed response.
        """
        if tools:
            processed_response = {
                "tool_calls": []
            }
            
            if response["output"]["message"]["content"]:
                for item in response["output"]["message"]["content"]:
                    if "toolUse" in item:
                        processed_response["tool_calls"].append({
                            "name": item["toolUse"]["name"],
                            "arguments": item["toolUse"]["input"]
                        })
            
            return processed_response
        
        response_body = json.loads(response['body'].read().decode())
        return response_body.get('completion', '')
    
    def _prepare_input(
            self,
            provider: str,
            model: str,
            prompt: str,
            model_kwargs: Optional[Dict[str, Any]] = {},
        ) -> Dict[str, Any]:
        """
            Prepares the input dictionary for the specified provider's model by mapping and renaming
            keys in the input based on the provider's requirements.

            Args:
                provider (str): The name of the service provider (e.g., "meta", "ai21", "mistral", "cohere", "amazon").
                model (str): The name or identifier of the model being used.
                prompt (str): The text prompt to be processed by the model.
                model_kwargs (Dict[str, Any]): Additional keyword arguments specific to the model's requirements.

            Returns:
                Dict[str, Any]: The prepared input dictionary with the correct keys and values for the specified provider.
        """

        input_body = {"prompt": prompt, **model_kwargs}
    
        provider_mappings = {
            "meta": {"max_tokens_to_sample": "max_gen_len"},
            "ai21": {"max_tokens_to_sample": "maxTokens", "top_p": "topP"},
            "mistral": {"max_tokens_to_sample": "max_tokens"},
            "cohere": {"max_tokens_to_sample": "max_tokens", "top_p": "p"},
        }
        
        if provider in provider_mappings:
            for old_key, new_key in provider_mappings[provider].items():
                if old_key in input_body:
                    input_body[new_key] = input_body.pop(old_key)
        
        if provider == "cohere" and "cohere.command-r" in model:
            input_body["message"] = input_body.pop("prompt")
        
        if provider == "amazon":
            input_body = {
                "inputText": prompt,
                "textGenerationConfig": {
                    "maxTokenCount": model_kwargs.get("max_tokens_to_sample"),
                    "topP": model_kwargs.get("top_p"),
                    "temperature": model_kwargs.get("temperature")
                }
            }
            input_body["textGenerationConfig"] = {k: v for k, v in input_body["textGenerationConfig"].items() if v is not None}
    
        return input_body

    def _convert_tool_format(self, original_tools):
        """
        Converts a list of tools from their original format to a new standardized format.

        Args:
            original_tools (list): A list of dictionaries representing the original tools, each containing a 'type' key and corresponding details.

        Returns:
            list: A list of dictionaries representing the tools in the new standardized format.
        """
        new_tools = []
        
        for tool in original_tools:
            if tool['type'] == 'function':
                function = tool['function']
                new_tool = {
                    "toolSpec": {
                        "name": function['name'],
                        "description": function['description'],
                        "inputSchema": {
                            "json": {
                                "type": "object",
                                "properties": {},
                                "required": function['parameters'].get('required', [])
                            }
                        }
                    }
                }
                
                for prop, details in function['parameters'].get('properties', {}).items():
                    new_tool["toolSpec"]["inputSchema"]["json"]["properties"][prop] = {
                        "type": details.get('type', 'string'),
                        "description": details.get('description', '')
                    }
                
                new_tools.append(new_tool)
        
        return new_tools

    def generate_response(
        self,
        messages: List[Dict[str, str]],
        tools: Optional[List[Dict]] = None,
        tool_choice: str = "auto",
    ):
        """
        Generate a response based on the given messages using AWS Bedrock.

        Args:
            messages (list): List of message dicts containing 'role' and 'content'.
            tools (list, optional): List of tools that the model can call. Defaults to None.
            tool_choice (str, optional): Tool choice method. Defaults to "auto".

        Returns:
            str: The generated response.
        """

        if tools:
            # Use converse method when tools are provided
            messages = [{"role": "user", "content": [{"text": message["content"]} for message in messages]}]
            inference_config = {"temperature": self.model_kwargs["temperature"], "maxTokens": self.model_kwargs["max_tokens_to_sample"], "topP": self.model_kwargs["top_p"]}
            tools_config = {"tools": self._convert_tool_format(tools)}

            response = self.client.converse(
                modelId=self.config.model,
                messages=messages,
                inferenceConfig=inference_config,
                toolConfig=tools_config
            )
        else:
            # Use invoke_model method when no tools are provided
            prompt = self._format_messages(messages)
            provider = self.model.split(".")[0]
            input_body = self._prepare_input(provider, self.config.model, prompt, **self.model_kwargs)
            body = json.dumps(input_body)

            response = self.client.invoke_model(
                body=body,
                modelId=self.model,
                accept='application/json',
                contentType='application/json'
            )

        return self._parse_response(response, tools)



from openai import OpenAI

from mem0.embeddings.base import EmbeddingBase


class OpenAIEmbedding(EmbeddingBase):
    def __init__(self, model="text-embedding-3-small"):
        self.client = OpenAI()
        self.model = model
        self.dims = 1536

    def embed(self, text):
        """
        Get the embedding for the given text using OpenAI.

        Args:
            text (str): The text to embed.

        Returns:
            list: The embedding vector.
        """
        text = text.replace("\n", " ")
        return (
            self.client.embeddings.create(input=[text], model=self.model)
            .data[0]
            .embedding
        )



from abc import ABC, abstractmethod


class EmbeddingBase(ABC):
    @abstractmethod
    def embed(self, text):
        """
        Get the embedding for the given text.

        Args:
            text (str): The text to embed.

        Returns:
            list: The embedding vector.
        """
        pass



from typing import Optional

from pydantic import BaseModel, Field, field_validator, model_validator


class QdrantConfig(BaseModel):
    host: Optional[str] = Field(None, description="Host address for Qdrant server")
    port: Optional[int] = Field(None, description="Port for Qdrant server")
    path: Optional[str] = Field(None, description="Path for local Qdrant database")
    url: Optional[str] = Field(None, description="Full URL for Qdrant server")
    api_key: Optional[str] = Field(None, description="API key for Qdrant server")

    @model_validator(mode="before")
    def check_host_port_or_path(cls, values):
        host, port, path, url, api_key = (
            values.get("host"),
            values.get("port"),
            values.get("path"),
            values.get("url"),
            values.get("api_key"),
        )
        if not path and not (host and port) and not (url and api_key):
            raise ValueError(
                "Either 'host' and 'port' or 'url' and 'api_key' or 'path' must be provided."
            )
        return values


class VectorStoreConfig(BaseModel):
    provider: str = Field(
        description="Provider of the vector store (e.g., 'qdrant', 'chromadb', 'elasticsearch')",
        default="qdrant",
    )
    config: QdrantConfig = Field(
        description="Configuration for the specific vector store",
        default=QdrantConfig(path="/tmp/qdrant"),
    )

    @field_validator("config")
    def validate_config(cls, v, values):
        provider = values.data.get("provider")
        if provider == "qdrant":
            return QdrantConfig(**v.model_dump())
        else:
            raise ValueError(f"Unsupported vector store provider: {provider}")



import importlib

from mem0.configs.llms.base import BaseLlmConfig


def load_class(class_type):
    module_path, class_name = class_type.rsplit(".", 1)
    module = importlib.import_module(module_path)
    return getattr(module, class_name)


class LlmFactory:
    provider_to_class = {
        "ollama": "mem0.llms.ollama.py.OllamaLLM",
        "openai": "mem0.llms.openai.OpenAILLM",
        "groq": "mem0.llms.groq.GroqLLM",
        "together": "mem0.llms.together.TogetherLLM",
        "aws_bedrock": "mem0.llms.aws_bedrock.AWSBedrockLLM",
        "litellm": "mem0.llms.litellm.LiteLLM",
    }

    @classmethod
    def create(cls, provider_name, config):
        class_type = cls.provider_to_class.get(provider_name)
        if class_type:
            llm_instance = load_class(class_type)
            base_config = BaseLlmConfig(**config)
            return llm_instance(base_config)
        else:
            raise ValueError(f"Unsupported Llm provider: {provider_name}")
        
class EmbedderFactory:
    provider_to_class = {
        "openai": "mem0.embeddings.openai.OpenAIEmbedding",
        "ollama": "mem0.embeddings.ollama.OllamaEmbedding",
        "huggingface": "mem0.embeddings.huggingface.HuggingFaceEmbedding"
    }

    @classmethod
    def create(cls, provider_name):
        class_type = cls.provider_to_class.get(provider_name)
        if class_type:
            embedder_instance = load_class(class_type)()
            return embedder_instance
        else:
            raise ValueError(f"Unsupported Embedder provider: {provider_name}")
        


import httpx
import os
import logging
import warnings
from typing import Optional, Dict, Any
from mem0.memory.setup import setup_config
from mem0.memory.telemetry import capture_client_event

logger = logging.getLogger(__name__)

# Setup user config
setup_config()


class MemoryClient:
    def __init__(self, api_key: Optional[str] = None, host: Optional[str] = None):
        """
        Initialize the Mem0 client.

        Args:
            api_key (Optional[str]): API Key from Mem0 Platform. Defaults to environment variable 'MEM0_API_KEY' if not provided.
            host (Optional[str]): API host URL. Defaults to 'https://api.mem0.ai/v1'.
        """
        self.api_key = api_key or os.getenv("MEM0_API_KEY")
        self.host = host or "https://api.mem0.ai/v1"
        self.client = httpx.Client(
            base_url=self.host,
            headers={"Authorization": f"Token {self.api_key}"},
        )
        self._validate_api_key()
        capture_client_event("client.init", self)

    def _validate_api_key(self):
        if not self.api_key:
            warnings.warn("API Key not provided. Please provide an API Key.")
        response = self.client.get("/memories/", params={"user_id": "test"})
        if response.status_code != 200:
            raise ValueError(
                "Invalid API Key. Please get a valid API Key from https://app.mem0.ai"
            )

    def add(
        self,
        data: str,
        user_id: Optional[str] = None,
        agent_id: Optional[str] = None,
        session_id: Optional[str] = None,
        metadata: Optional[Dict[str, Any]] = None,
        filters: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        """
        Create a new memory.

        Args:
            data (str): The data to be stored in the memory.
            user_id (Optional[str]): User ID to save the memory specific to a user. Defaults to None.
            agent_id (Optional[str]): Agent ID for agent-specific memory. Defaults to None.
            session_id (Optional[str]): Run ID to save memory for a specific session. Defaults to None.
            metadata (Optional[Dict[str, Any]]): Metadata to be saved with the memory. Defaults to None.
            filters (Optional[Dict[str, Any]]): Filters to apply to the memory. Defaults to None.

        Returns:
            Dict[str, Any]: The response from the server.
        """
        capture_client_event("client.add", self)
        payload = {"text": data}
        if metadata:
            payload["metadata"] = metadata
        if filters:
            payload["filters"] = filters
        if user_id:
            payload["user_id"] = user_id
        if agent_id:
            payload["agent_id"] = agent_id
        if session_id:
            payload["run_id"] = session_id

        response = self.client.post("/memories/", json=payload, timeout=60)
        if response.status_code != 200:
            logger.error(response.json())
            raise ValueError(f"Failed to add memory. Response: {response.json()}")
        return response.json()

    def get(self, memory_id: str) -> Dict[str, Any]:
        """
        Get a memory by ID.

        Args:
            memory_id (str): Memory ID.

        Returns:
            Dict[str, Any]: The memory data.
        """
        capture_client_event("client.get", self)
        response = self.client.get(f"/memories/{memory_id}/")
        return response.json()

    def get_all(
        self,
        user_id: Optional[str] = None,
        agent_id: Optional[str] = None,
        session_id: Optional[str] = None,
        limit: int = 100,
    ) -> Dict[str, Any]:
        """
        Get all memories.

        Args:
            user_id (Optional[str]): User ID to filter memories. Defaults to None.
            agent_id (Optional[str]): Agent ID to filter memories. Defaults to None.
            session_id (Optional[str]): Run ID to filter memories. Defaults to None.
            limit (int): Number of memories to return. Defaults to 100.

        Returns:
            Dict[str, Any]: The list of memories.
        """
        params = {
            "user_id": user_id,
            "agent_id": agent_id,
            "run_id": session_id,
            "limit": limit,
        }
        response = self.client.get(
            "/memories/", params={k: v for k, v in params.items() if v is not None}
        )
        capture_client_event(
            "client.get_all", self, {"filters": len(params), "limit": limit}
        )
        return response.json()

    def search(
        self,
        query: str,
        user_id: Optional[str] = None,
        agent_id: Optional[str] = None,
        session_id: Optional[str] = None,
        limit: int = 100,
        filters: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        """
        Search memories.

        Args:
            query (str): Query to search for in the memories.
            user_id (Optional[str]): User ID to filter memories. Defaults to None.
            agent_id (Optional[str]): Agent ID to filter memories. Defaults to None.
            session_id (Optional[str]): Run ID to filter memories. Defaults to None.
            limit (int): Number of memories to return. Defaults to 100.
            filters (Optional[Dict[str, Any]]): Filters to apply to the search. Defaults to None.

        Returns:
            Dict[str, Any]: The search results.
        """
        payload = {
            "text": query,
            "limit": limit,
            "filters": filters,
            "user_id": user_id,
            "agent_id": agent_id,
            "run_id": session_id,
        }
        response = self.client.post("/memories/search/", json=payload)
        capture_client_event("client.search", self, {"limit": limit})
        return response.json()

    def update(self, memory_id: str, data: str) -> Dict[str, Any]:
        """
        Update a memory by ID.

        Args:
            memory_id (str): Memory ID.
            data (str): Data to update in the memory.

        Returns:
            Dict[str, Any]: The response from the server.
        """
        capture_client_event("client.update", self)
        response = self.client.put(f"/memories/{memory_id}/", json={"text": data})
        return response.json()

    def delete(self, memory_id: str) -> Dict[str, Any]:
        """
        Delete a memory by ID.

        Args:
            memory_id (str): Memory ID.

        Returns:
            Dict[str, Any]: The response from the server.
        """
        capture_client_event("client.delete", self)
        response = self.client.delete(f"/memories/{memory_id}/")
        return response.json()

    def delete_all(
        self,
        user_id: Optional[str] = None,
        agent_id: Optional[str] = None,
        session_id: Optional[str] = None,
    ) -> Dict[str, Any]:
        """
        Delete all memories.

        Args:
            user_id (Optional[str]): User ID to filter memories. Defaults to None.
            agent_id (Optional[str]): Agent ID to filter memories. Defaults to None.
            session_id (Optional[str]): Run ID to filter memories. Defaults to None.

        Returns:
            Dict[str, Any]: The response from the server.
        """
        params = {"user_id": user_id, "agent_id": agent_id, "run_id": session_id}
        response = self.client.delete(
            "/memories/", params={k: v for k, v in params.items() if v is not None}
        )
        capture_client_event("client.delete_all", self, {"params": len(params)})
        return response.json()

    def history(self, memory_id: str) -> Dict[str, Any]:
        """
        Get history of a memory by ID.

        Args:
            memory_id (str): Memory ID.

        Returns:
            Dict[str, Any]: The memory history.
        """
        response = self.client.get(f"/memories/{memory_id}/history/")
        capture_client_event("client.history", self)
        return response.json()

    def reset(self):
        """
        Reset the client. (Not implemented yet)
        """
        raise NotImplementedError("Reset is not implemented yet")

    def chat(self):
        """
        Start a chat with the Mem0 AI. (Not implemented yet)
        """
        raise NotImplementedError("Chat is not implemented yet")



import os
import shutil
import logging
from typing import Optional

from pydantic import BaseModel, Field
from qdrant_client import QdrantClient
from qdrant_client.models import (
    Distance,
    FieldCondition,
    Filter,
    MatchValue,
    PointIdsList,
    PointStruct,
    Range,
    VectorParams,
)

from mem0.vector_stores.base import VectorStoreBase


class QdrantConfig(BaseModel):
    host: Optional[str] = Field(None, description="Host address for Qdrant server")
    port: Optional[int] = Field(None, description="Port for Qdrant server")
    path: Optional[str] = Field(None, description="Path for local Qdrant database")


class Qdrant(VectorStoreBase):
    def __init__(
        self,
        client=None,
        host="localhost",
        port=6333,
        path=None,
        url=None,
        api_key=None,
    ):
        """
        Initialize the Qdrant vector store.

        Args:
            client (QdrantClient, optional): Existing Qdrant client instance. Defaults to None.
            host (str, optional): Host address for Qdrant server. Defaults to "localhost".
            port (int, optional): Port for Qdrant server. Defaults to 6333.
            path (str, optional): Path for local Qdrant database. Defaults to None.
            url (str, optional): Full URL for Qdrant server. Defaults to None.
            api_key (str, optional): API key for Qdrant server. Defaults to None.
        """
        if client:
            self.client = client
        else:
            params = {}
            if path:
                params["path"] = path
                if os.path.exists(path) and os.path.isdir(path):
                    shutil.rmtree(path)
            if api_key:
                params["api_key"] = api_key
            if url:
                params["url"] = url
            if host and port:
                params["host"] = host
                params["port"] = port
            self.client = QdrantClient(**params)

    def create_col(self, name, vector_size, distance=Distance.COSINE):
        """
        Create a new collection.

        Args:
            name (str): Name of the collection.
            vector_size (int): Size of the vectors to be stored.
            distance (Distance, optional): Distance metric for vector similarity. Defaults to Distance.COSINE.
        """
        # Skip creating collection if already exists
        response = self.list_cols()
        for collection in response.collections:
            if collection.name == name:
                logging.debug(f"Collection {name} already exists. Skipping creation.")
                return

        self.client.create_collection(
            collection_name=name,
            vectors_config=VectorParams(size=vector_size, distance=distance),
        )

    def insert(self, name, vectors, payloads=None, ids=None):
        """
        Insert vectors into a collection.

        Args:
            name (str): Name of the collection.
            vectors (list): List of vectors to insert.
            payloads (list, optional): List of payloads corresponding to vectors. Defaults to None.
            ids (list, optional): List of IDs corresponding to vectors. Defaults to None.
        """
        points = [
            PointStruct(
                id=idx if ids is None else ids[idx],
                vector=vector,
                payload=payloads[idx] if payloads else {},
            )
            for idx, vector in enumerate(vectors)
        ]
        self.client.upsert(collection_name=name, points=points)

    def _create_filter(self, filters):
        """
        Create a Filter object from the provided filters.

        Args:
            filters (dict): Filters to apply.

        Returns:
            Filter: The created Filter object.
        """
        conditions = []
        for key, value in filters.items():
            if isinstance(value, dict) and "gte" in value and "lte" in value:
                conditions.append(
                    FieldCondition(
                        key=key, range=Range(gte=value["gte"], lte=value["lte"])
                    )
                )
            else:
                conditions.append(
                    FieldCondition(key=key, match=MatchValue(value=value))
                )
        return Filter(must=conditions) if conditions else None

    def search(self, name, query, limit=5, filters=None):
        """
        Search for similar vectors.

        Args:
            name (str): Name of the collection.
            query (list): Query vector.
            limit (int, optional): Number of results to return. Defaults to 5.
            filters (dict, optional): Filters to apply to the search. Defaults to None.

        Returns:
            list: Search results.
        """
        query_filter = self._create_filter(filters) if filters else None
        hits = self.client.search(
            collection_name=name,
            query_vector=query,
            query_filter=query_filter,
            limit=limit,
        )
        return hits

    def delete(self, name, vector_id):
        """
        Delete a vector by ID.

        Args:
            name (str): Name of the collection.
            vector_id (int): ID of the vector to delete.
        """
        self.client.delete(
            collection_name=name,
            points_selector=PointIdsList(
                points=[vector_id],
            ),
        )

    def update(self, name, vector_id, vector=None, payload=None):
        """
        Update a vector and its payload.

        Args:
            name (str): Name of the collection.
            vector_id (int): ID of the vector to update.
            vector (list, optional): Updated vector. Defaults to None.
            payload (dict, optional): Updated payload. Defaults to None.
        """
        point = PointStruct(id=vector_id, vector=vector, payload=payload)
        self.client.upsert(collection_name=name, points=[point])

    def get(self, name, vector_id):
        """
        Retrieve a vector by ID.

        Args:
            name (str): Name of the collection.
            vector_id (int): ID of the vector to retrieve.

        Returns:
            dict: Retrieved vector.
        """
        result = self.client.retrieve(
            collection_name=name, ids=[vector_id], with_payload=True
        )
        return result[0] if result else None

    def list_cols(self):
        """
        List all collections.

        Returns:
            list: List of collection names.
        """
        return self.client.get_collections()

    def delete_col(self, name):
        """
        Delete a collection.

        Args:
            name (str): Name of the collection to delete.
        """
        self.client.delete_collection(collection_name=name)

    def col_info(self, name):
        """
        Get information about a collection.

        Args:
            name (str): Name of the collection.

        Returns:
            dict: Collection information.
        """
        return self.client.get_collection(collection_name=name)

    def list(self, name, filters=None, limit=100):
        """
        List all vectors in a collection.

        Args:
            name (str): Name of the collection.
            limit (int, optional): Number of vectors to return. Defaults to 100.

        Returns:
            list: List of vectors.
        """
        query_filter = self._create_filter(filters) if filters else None
        result = self.client.scroll(
            collection_name=name,
            scroll_filter=query_filter,
            limit=limit,
            with_payload=True,
            with_vectors=False,
        )
        return result
